/*
 *    Virtual Steganographic Laboratory (VSL)
 *    Copyright (C) 2008-2011  M. Wegrzyn <bend-up@users.sourceforge.net>

 *    This file is part of Virtual Steganographic Laboratory (VSL).

 *    VSL is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.

 *    VSL is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.

 *    You should have received a copy of the GNU General Public License
 *    along with VSL.  If not, see <http://www.gnu.org/licenses/>.
 */
package pl.edu.zut.wi.vsl.app.gui.panels;

import pl.edu.zut.wi.vsl.app.utils.*;
import java.io.File;
import java.util.Arrays;
import java.util.List;
import javax.swing.filechooser.FileFilter;
import pl.edu.zut.wi.vsl.app.Main;

/**
 * Image filter for <code>JFileChooser</code> used for saving image dialogs.
 * 
 * @author Michal Wegrzyn
 */
public class DisplayImageFilter extends FileFilter {

    private final List<String> formats;
    private final String description;

    public DisplayImageFilter() {

        String[] supported = Main.getSupportedOutputFormats();
        formats = Arrays.asList(supported);
        StringBuilder desc = new StringBuilder();
        for (int i = 0; i < supported.length; ++i) {
            desc.append("*.");
            desc.append(supported[i]);
            desc.append(" ");
        }
        description = desc.toString();
    }

    /**
     * Method deciding to display (or not) given file in 
     * <code>JFileChooser</code>. This particular <code>FileFilter</code>
     * accepts all directories and all gif, jpg, tiff, or png files.
     * @param f File to analyse
     * @return <code>true</code> if file can be displayed,
     * otherwise <code>false</code>
     */
    public boolean accept(File f) {
        if (f.isDirectory()) {
            return true;
        }
        String extension = FileUtil.getFileFormat(f.getAbsolutePath());
        if (extension == null) {
            return false;
        } else {
            extension = extension.toLowerCase();
            

            return formats.contains(extension);
        }

    }

    /**
     * Returns the description of this filter.
     */
    public String getDescription() {
        return description;
    }
}
