/*
 *    Virtual Steganographic Laboratory (VSL)
 *    Copyright (C) 2008-2011  M. Wegrzyn <bend-up@users.sourceforge.net>

 *    This file is part of Virtual Steganographic Laboratory (VSL).

 *    VSL is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.

 *    VSL is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.

 *    You should have received a copy of the GNU General Public License
 *    along with VSL.  If not, see <http://www.gnu.org/licenses/>.
 */
package pl.edu.zut.wi.vsl.app.gui.panels;

import pl.edu.zut.wi.vsl.app.MainWindow;

/**
 * Panel for displaying content of throwed errors in clear way.
 * 
 * @author Michal Wegrzyn
 */
public class ErrorPane extends javax.swing.JPanel {

    /** Message for error */
    private final String message;
    /** Content of throwed error */
    private final String error;
    
    /**
     * Creates new form ErrorPane.
     * @param msg Message for error
     * @param err Content of throwed error
     */
    public ErrorPane(String msg, String err) {
        message = msg;
        error = err;
        initComponents();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        messageLabel = new javax.swing.JLabel(message);
        messageLabel.setToolTipText(message);
        errorScrollPane = new javax.swing.JScrollPane();
        errorScrollPane.getVerticalScrollBar().setUnitIncrement(MainWindow.getDefaultScrollIncrement());
        errorScrollPane.getHorizontalScrollBar().setUnitIncrement(MainWindow.getDefaultScrollIncrement());
        errorScrollPane.getVerticalScrollBar().setBlockIncrement(0);
        errorScrollPane.getHorizontalScrollBar().setBlockIncrement(0);
        errorTextArea = new javax.swing.JTextArea();
        errorTextArea.setText(error);
        errorTextArea.setCaretPosition(0);
        errorTraceLabel = new javax.swing.JLabel();

        errorTextArea.setColumns(20);
        errorTextArea.setEditable(false);
        errorTextArea.setRows(5);
        errorScrollPane.setViewportView(errorTextArea);

        errorTraceLabel.setText("Error trace:");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(messageLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 549, Short.MAX_VALUE)
            .addGroup(layout.createSequentialGroup()
                .addComponent(errorTraceLabel)
                .addContainerGap())
            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addComponent(errorScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 549, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(messageLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(errorTraceLabel)
                .addContainerGap(152, Short.MAX_VALUE))
            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(layout.createSequentialGroup()
                    .addContainerGap(45, Short.MAX_VALUE)
                    .addComponent(errorScrollPane, javax.swing.GroupLayout.PREFERRED_SIZE, 151, javax.swing.GroupLayout.PREFERRED_SIZE)))
        );
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane errorScrollPane;
    private javax.swing.JTextArea errorTextArea;
    private javax.swing.JLabel errorTraceLabel;
    private javax.swing.JLabel messageLabel;
    // End of variables declaration//GEN-END:variables

}
