/*
 *    Virtual Steganographic Laboratory (VSL)
 *    Copyright (C) 2008-2011  M. Wegrzyn <bend-up@users.sourceforge.net>

 *    This file is part of Virtual Steganographic Laboratory (VSL).

 *    VSL is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.

 *    VSL is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.

 *    You should have received a copy of the GNU General Public License
 *    along with VSL.  If not, see <http://www.gnu.org/licenses/>.
 */
package pl.edu.zut.wi.vsl.app.modules;

import java.io.Serializable;

/**
 * Parameter for the module.
 *
 * @author Michal Wegrzyn
 */
public class ModuleParameter implements Serializable {

    /** For serialization. */
    private static final long serialVersionUID = -4863178801181714313L;
    
    private String value;
    private String description;
    private final String defaultValue;
    private final String name;

    public ModuleParameter(String name, String value, String description) {
        this.value = value;
        this.defaultValue = value;
        this.name = name;
        this.description = description;
    }

    ModuleParameter(ModuleParameter param) {
        this(param.getName(), param.getValue(), param.getDescription());
    }

    public void reset() {
        this.value = this.defaultValue;
    }

    public String getDefaultValue() {
        return defaultValue;
    }

    public String getName() {
        return name;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getValue() {
        return value;
    }

    public void setValue(String value) {
        this.value = value;
    }

}
