/*
 *    Virtual Steganographic Laboratory (VSL)
 *    Copyright (C) 2008-2011  M. Wegrzyn <bend-up@users.sourceforge.net>

 *    This file is part of Virtual Steganographic Laboratory (VSL).

 *    VSL is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.

 *    VSL is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.

 *    You should have received a copy of the GNU General Public License
 *    along with VSL.  If not, see <http://www.gnu.org/licenses/>.
 */
package pl.edu.zut.wi.vsl.app.modules;

import java.io.Serializable;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;
import org.apache.log4j.Logger;
import pl.edu.zut.wi.vsl.app.MainWindow;
import pl.edu.zut.wi.vsl.app.VslException;
import pl.edu.zut.wi.vsl.commons.StegoPackage;
import pl.edu.zut.wi.vsl.commons.steganalysis.SteganalysisException;
import pl.edu.zut.wi.vsl.commons.steganalysis.SteganalyticTechnique;

/**
 * Module for stegananalysers.
 *
 * @author Michal Wegrzyn
 */
public class SteganalyticModule extends VslModule implements Serializable {

    /** For serialization. */
    private static final long serialVersionUID = -5387143434021807411L;

    private final static Logger logger = Logger.getLogger(SteganalyticModule.class);
    
    public SteganalyticModule(String name, String description, String clazz) {
        super(name, description, clazz, ModuleType.Steganalytic);
    }

    public SteganalyticModule(SteganalyticModule v) {
        super(v);
    }

    @Override
    public StegoPackage execute(StegoPackage cp) throws VslException {
        int paramIndex = getLoopNumber();

        try {
            SteganalyticTechnique a = (SteganalyticTechnique)
                    (Class.forName(getModuleClass()).newInstance());
            LinkedHashMap<String, String> results = a.analyse(cp.getImage(),
                    getParameterMap(paramIndex));
            Set<Map.Entry<String, String>> s = results.entrySet();

            // log results from steganalysis
            for (Map.Entry<String, String> me : s) {
                MainWindow.getLogger().logMessage(me.getKey() + ": "  + me.getValue());
                logger.info(getModuleClass() + ": " + me.getKey() + ": "  + me.getValue());
            }

            cp.setInfo(results.get("result"));

        } catch (SteganalysisException e) {
            throw new VslException("Exception during execution", e);
        } catch (ClassNotFoundException e ) {
            throw new VslException("Exception during execution", e);
        } catch (InstantiationException e) {
            throw new VslException("Exception during execution", e);
        } catch ( IllegalAccessException e) {
            throw new VslException("Exception during execution", e);
        }

        return cp;
    }

}
