/*
 *    Virtual Steganographic Laboratory (VSL)
 *    Copyright (C) 2008-2011  M. Wegrzyn <bend-up@users.sourceforge.net>

 *    This file is part of Virtual Steganographic Laboratory (VSL).

 *    VSL is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.

 *    VSL is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.

 *    You should have received a copy of the GNU General Public License
 *    along with VSL.  If not, see <http://www.gnu.org/licenses/>.
 */
package pl.edu.zut.wi.vsl.commons.steganalysis;

import java.util.LinkedHashMap;
import pl.edu.zut.wi.vsl.commons.StegoImage;

/**
 * Interface representing steganalytic technique.
 * 
 * @author Michal Wegrzyn
 */
public interface SteganalyticTechnique {

    /**
     * Method which analizes given image to help find out if any object
     * was embedded.
     * @param StegoImage image to analyse.
     * @param options parameters for technique
     * @return report which tells or helps to find out is any object embedded
     * in examined image.
     * @throws pl.ps.wi.vsl.commons.steganalysis.SteganalysisException
     */
    public LinkedHashMap<String, String> analyse(StegoImage img, 
                                        LinkedHashMap<String, String> options)
                                        throws SteganalysisException;
}
